import { withProfiler } from '@sentry/react'
import { Text } from 'holi-bricks/components/text'
import type React from 'react'
import { useEffect } from 'react'
import { Trans, useTranslation } from 'react-i18next'
import { Platform, StyleSheet, View } from 'react-native'
import Animated, {
  Easing,
  useAnimatedStyle,
  useSharedValue,
  withRepeat,
  withSequence,
  withSpring,
  withTiming,
} from 'react-native-reanimated'

import { useLoggedInUser } from '@holi/core/auth/hooks/useLoggedInUser'
import { useIsFocused } from '@holi/core/helpers/useIsFocused'
import { HoliTextLink } from '@holi/core/navigation/components/HoliTextLink'
import AnnouncementBanner from '@holi/core/screens/homeFeed/components/AnnouncementBanner'
import HandWaveFilledPng from '@holi/ui/assets/img/icons/hand-wave-filled.png'
import HandWaveFilled from '@holi/ui/assets/img/icons/hand-wave-filled.svg'
import HoliBox from '@holi/ui/components/atoms/HoliBox'
import { HoliGap } from '@holi/ui/components/atoms/HoliGap'
import HoliLocalImage from '@holi/ui/components/atoms/HoliLocalImage'
import ProfileCompletionCard from '@holi/core/screens/userprofile/components/ProfileCompletionCard'

const HomeFeedHeader: React.FC = () => {
  const { t } = useTranslation()
  const isFocused = useIsFocused()

  const { user, isLoading } = useLoggedInUser()

  const bannerShowCutoff = new Date(2024, 3, 12, 0, 0, 0)

  const rotation = useSharedValue(0)
  const rotationStyle = useAnimatedStyle(
    () => ({
      transform: [{ rotate: `${rotation.value}deg` }],
    }),
    [rotation]
  )
  useEffect(() => {
    rotation.value = withRepeat(
      withSequence(
        // delay
        withTiming(0, { duration: 1500 }),
        // waving
        withTiming(20, { easing: Easing.inOut(Easing.sin) }),
        withTiming(-20, { easing: Easing.inOut(Easing.sin) }),
        withTiming(26, { easing: Easing.inOut(Easing.sin) }),
        withTiming(-26, { easing: Easing.inOut(Easing.sin) }),
        withTiming(26, { easing: Easing.inOut(Easing.sin) }),
        withTiming(-26, { easing: Easing.inOut(Easing.sin) }),
        withTiming(20, { easing: Easing.inOut(Easing.sin) }),
        withTiming(-20, { easing: Easing.inOut(Easing.sin) }),
        withSpring(0, { damping: 8, stiffness: 70 }),
        // delay
        withTiming(0, { duration: 15000 })
      ),
      3,
      true
    )
  }, [rotation])

  return (
    <View>
      {new Date() < bannerShowCutoff && (
        <>
          <HoliGap size="s" />
          <AnnouncementBanner>
            <Text size="md">
              <Trans t={t} i18nKey="homeFeed.banner.privacyPolicyUpdated">
                <HoliTextLink
                  label={t('profile.settings.legalDocuments.privacyPolicy')}
                  href="/profile/settings/legalDocuments/document?type=privacyPolicy"
                  size="md"
                >
                  {t('profile.settings.legalDocuments.privacyPolicy')}
                </HoliTextLink>
              </Trans>
            </Text>
          </AnnouncementBanner>
        </>
      )}

      {/*Headline */}
      <HoliBox padding={[32, 0, 8, 0]} layout={styles.headlineContainer} testID="home-feed-insights-heading">
        <Text size="3xl" headingLevel="2">
          {user?.firstName ? t('homeFeed.heading.loggedIn', { firstname: user?.firstName }) : t('homeFeed.heading')}
        </Text>
        {/* start waving animation after user state has been loaded and the screen is focused/active */}
        <Animated.View style={[!isLoading && isFocused && rotationStyle, { transformOrigin: 'center bottom' }]}>
          <HoliLocalImage
            imageSource={Platform.OS === 'android' ? HandWaveFilledPng : HandWaveFilled}
            width={40}
            height={40}
            label={''}
            style={styles.handwaveFilled}
            isSvg={Platform.OS !== 'android'}
          />
        </Animated.View>
      </HoliBox>
      <ProfileCompletionCard style={styles.container} />
    </View>
  )
}

export default withProfiler(HomeFeedHeader, { name: 'HomeFeedHeader' })

const styles = StyleSheet.create({
  headlineContainer: {
    flexDirection: 'row',
    alignItems: 'center',
  },
  handwaveFilled: {
    paddingLeft: 8,
  },
  container: {
    marginTop: 16,
  },
})
